/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.gui.treetable;

import isac.interfaces.ICalcIterator;
import isac.util.CalcChanged;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Position;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

/**
 * 
 * @author Mario Hochreiter
 * @version 0.1 18/09/2003
 * @version 1.1
 * @author Alois Kirchsteiger 2005
 */
public class CalculationModel extends AbstractTreeModel implements Serializable {
	public static final long serialVersionUID = 502L;
	
	private static final Logger logger = Logger
            .getLogger(CalculationModel.class.getName());

    protected Object root;

    protected CalcModelHierarchy hierarchy_;

    /**
     * @param root
     *            the root element of the <code>CalculationModel</code>
     */
    public CalculationModel(CalcModelHierarchy h) {
        super(h.getRoot());
        root = h.getRoot();
        hierarchy_ = h;
    }

    /**
     * @return The root element of the <code>CalculationTree</code>
     * @throws IllegalArgumenException
     */
    public Object getRoot() {
        if (root instanceof CalcModelNode) {
            return root;
        }
        throw new IllegalArgumentException(
                "CalculationModel can only handle objects"
                        + "of type CalcModelHierarchy");
    }

    /**
     * @see javax.swing.tree.TreeModel#getChildCount(java.lang.Object)
     */
    public int getChildCount(Object parent) {
        return hierarchy_.getChildCount((CalcModelNode) parent);
    }

    /**
     * @see javax.swing.tree.TreeModel#getChild(java.lang.Object, int)
     */
    public Object getChild(Object parent, int index) {
        return ((CalcModelNode) parent).getChild(index);
        //    return hierarchy_.getChild((CalcModelNode)parent, index);
    }

    public void setCalcModelHierarchy(CalcModelHierarchy h) {
        this.hierarchy_ = h;
    }

     /**
     * updates the tree from a calcChangedEvent
     * 
     * WN0504DI the ONLY call in <code>Worksheet#calcChanged</code> assumes
     * that ALL formulae between
     * <code>CalcChanged#getLastUnchangedFormula</code> and
     * <code>CalcChanged#getLastGeneratedFormula</code> are displayed on
     * the <code>Worksheet</code>.
     */
    public void calcChanged(CalcChanged e) {
        try {
        	removeInsertionNode();
            //			something has been deleted           
            if (e.getLastDeletedFormula().compareToIterator(e.getLastUnchangedFormula()) != 0) {
            	ICalcIterator from = e.getLastUnchangedFormula();
            	ICalcIterator to = e.getLastDeletedFormula();
            	removeFromTo( from, to );
            }

            if (e.getLastGeneratedFormula() != null) {
            	ICalcIterator from = e.getLastUnchangedFormula();
            	ICalcIterator to = e.getLastGeneratedFormula();
            	insertFromTo( from, to );
            }
            updateInsertionNode();
        } catch (RemoteException ex) {
        	System.out.println("Remote Exception occured in calcChanged of the CalculationModel !!");
            ex.printStackTrace();
        }
    }

    /**
     * informs the tree to insert one node that is specified by a Object[] oa
     * where oa[0] = path to the parent oa[1] = int[1] with the index of the
     * inserted node as only element oa[2] = Object[1] with the inserted node as
     * only element
     */
    private void insert( CalcFormula formula ) {
        Object[] oa = hierarchy_.insert( formula );
        
        fireTreeNodesInserted(this, (Object[]) oa[0], (int[]) oa[1],
                (Object[]) oa[2]);
        fireTreeStructureChanged(this, (Object[]) oa[0], (int[]) oa[1],
                (Object[]) oa[2]);    	
    } 
    
    /**
     * informs the tree to insert nodes between the two iterators
     * 
     * @author Nebojsa Simic
     */    
    private void insertFromTo( ICalcIterator from, ICalcIterator to ) throws RemoteException {
        Vector calc_formulae = null;
		calc_formulae = from.getFormulaeFromTo( to, new Integer(0), false  );

		if( calc_formulae == null )
			return;
		
    	Iterator vector_it = calc_formulae.iterator();
    	
    	while( vector_it.hasNext() ) {
    		insert( (CalcFormula) vector_it.next() );
    	}    	
    }
 
 
    /**
     * informs the tree to remove nodes between the two iterators
     * 
     * @author Nebojsa Simic
     */    
    private void removeFromTo( ICalcIterator from, ICalcIterator to ) throws RemoteException {
    	Position from_pos = null;
    	Position to_pos = null;
		from_pos = from.getPosition();
		to_pos = to.getPosition();
		
		if( from_pos == null )
			System.out.println("From position is null");
		else
			System.out.println("From position is " + from_pos.toString() );
		
		if( to_pos == null )
			System.out.println("To position is null");
		else
			System.out.println("To position is " + to_pos.toString() );
		

		Object[] oda = hierarchy_.deleteFromTo( (Position)from_pos, (Position)to_pos );	
		
		Object[] indices = (Object[]) oda[1];
		int[] int_indices = new int[indices.length];
		
		for( int i = 0; i < indices.length;i ++)
			int_indices[i] = ((Integer)indices[i]).intValue();
	
		if( indices.length == 0 ) // Yet another sanity check !! In reality this should not happen !!
			return; 
		
        fireTreeNodesRemoved(this, (Object[]) oda[0], int_indices,
                (Object[]) oda[2]);
        fireTreeStructureChanged(this, (Object[]) oda[0], int_indices,
                (Object[]) oda[2]);
    }
    
    private void removeInsertionNode() {
        Object[] oa = hierarchy_.removeInsertionNode();
        
        if( oa == null )
	    	return;
        
        fireTreeNodesRemoved(this, (Object[]) oa[0], (int[]) oa[1],
                (Object[]) oa[2]);
        fireTreeStructureChanged(this, (Object[]) oa[0], (int[]) oa[1],
                (Object[]) oa[2]);

    }

    private void updateInsertionNode() {    
	    Object[] oa = hierarchy_.updateInsertionNode();
	    
	    if( oa == null )
	    	return;
	    
	    fireTreeNodesInserted(this, (Object[]) oa[0], (int[]) oa[1],
	            (Object[]) oa[2]);
	    fireTreeStructureChanged(this, (Object[]) oa[0], (int[]) oa[1],
	            (Object[]) oa[2]);
    }

	public void valueForPathChanged(TreePath path, Object newValue) {
		super.valueForPathChanged(path, newValue);
	
		firePathUpdated( path, newValue );
	}
    

    

}